/**
 * The maximum length of an array.
 *
 * According to the [ECMA-262](https://tc39.es/ecma262/#array-index):
 *     0 <= array.length <= 2^32 - 1
 *
 * If this value should be updated, please
 * submit a pull request with details supporting a new max length.
 */
export declare const MAX_ARRAY_LENGTH = 4294967295;
/**
 * The maximum number of arguments that can be safely accepted by a function.
 *
 * According to the [ECMA-262](https://tc39.es/ecma262/#sec-list-and-record-specification-type), there is no maximum. In practice, different
 * environments impose their own limit ([source](https://stackoverflow.com/questions/22747068/is-there-a-max-number-of-arguments-javascript-functions-can-accept)).
 *
 * This value was chosen through research, local testing and iteration. The actual number of arguments
 * accepted in an environment may be larger but hopefully not smaller. If errors are raised,
 * please submit a bug with your environment details and a new max length.
 */
export declare const MAX_SAFE_ARGUMENT_LENGTH = 32767;
/**
 * Check if a value is an Array or TypedArray.
 *
 * @param obj - The value to check.
 *
 * @returns `true` if an Array or TypedArray, otherwise `false`.
 */
export declare function isArray<T = any>(obj: unknown): obj is T[];
/**
 * Check if a value is a TypedArray.
 *
 * See [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/TypedArray)
 * for more details.
 *
 * @param obj - The value to check.
 *
 * @returns `true` if a TypedArray, otherwise `false`.
 */
export declare const isTypedArray: <T = any>(obj: unknown) => obj is T[];
/**
 * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.
 *
 * Contrary to Array.splice, this will not throw an error if too many new elements are given as arguments.
 *
 * @param array - The array to splice.
 * @param start - The zero-based location in the array from which to start removing elements.
 * @param count - The number of elements to remove.
 * @param elements - The new elements to splice in.
 *
 * @returns - An array containing the elements that were deleted.
 */
export declare function splice<T>(array: T[], start?: number, count?: number, elements?: Iterable<T>): T[];
