import { CombineFn } from '..';
import { SegmentTree } from './segmentTree';
/**
 * A {@link SegmentTree} with entries stored in in-order traversal.
 * Inspired by [Tristan Hume's IForestIndex](https://thume.ca/2021/03/14/iforests) ([github](https://github.com/trishume/gigatrace))
 */
export declare class InOrderSegmentTree<T> implements SegmentTree<T> {
    /**
     * The maximum amount of elements that can be added.
     *
     * n elements require 2n memory.
     */
    static readonly MAX_SIZE: number;
    /**
     * The set of elements and aggregation nodes for the tree
     */
    protected array: T[];
    /**
     * The function used to aggregate elements
     */
    protected combine: CombineFn<T>;
    /**
     * Construct a new segment tree
     *
     * @param combine - The function used to aggregate segment information
     * @param elements - A set of elements to add into the initial tree
     */
    constructor(combine: CombineFn<T>, elements?: Iterable<T>);
    clear(): void;
    pop(): T | undefined;
    push(element: T): number;
    query(min: number, max: number): T;
    get size(): number;
    /**
     * Return an iterator through the tree's elements
     */
    [Symbol.iterator](): Iterator<T>;
    update(min: number, max: number, operation: (element: T, index: number) => T): void;
    /**
     * A helper method used to build the tree
     *
     * @param elements The initial set of elements to add into the tree
     */
    protected build(elements: Iterable<T>): void;
    /**
     * A helper method to update complete aggregation nodes for an index
     */
    protected set(index: number, element: T): T;
}
